local BEC = BuffEnough

SLASH_BECCMD1, SLASH_BECCMD2 = "/bec", "/buffenoughclassic"

-- TODO: Add check option for Paladin Seals.

local function PrintHelp()
	print("===================")
	print("Buff Enough Classic")
	print("===================")
	print("Slash Commands: |cFFFFFF00" .. SLASH_BECCMD1 .. ", " .. SLASH_BECCMD2.."|r")
	print("Commands:")
	print("|cFFFFFF00config|r")
	print("Display the current configuration.")
	print("|cFFFFFF00locked <true/false>|r")
	print("Locks or unlocks the frame for dragging.")
	print("|cFFFFFF00size <small/normal/large>|r")
	print("Sets the size of the indicator box.")	
	print("|cFFFFFF00ignoreoptional <true|false>|r")
	print("Ignores missing optional buffs.")
	print("|cFFFFFF00ignoreweak <true|false>|r")
	print("Ignores weak buffs.")
	print("|cFFFFFF00checkthorns <true|false>|r")
	print("Enables/disables checking of Thorns if there is a Druid in the group.")	
	print("===================")
end

function BEC.ParseFauxBufferRequest(parameters)
	local items = BEC:SplitString(parameters)

	local classID = BEC:GetClassID(items[1])

	if classID ~= 0 then
		local level = tonumber(items[2])

		if level > 0 and level <= BEC.Game_MaxLevel then
			BEC:AddFauxBuffer(items[1], classID, level)
		end
	else
		print("BEC: Error unknown class.")
	end
end

function BEC.ParsePlayerOverride(parameters)
	local items = BEC:SplitString(parameters)

	if strlower(items[1]) == "class" then
		local classID = BEC:GetClassID(items[2])

		if classID > 0 then
			BEC.OverrideClass = items[2]
			BEC.OverrideClassID = classID
		end
	elseif strlower(items[1]) == "level" then
		local level = tonumber(items[2])

		if level > 0 and level <= BEC.Game_MaxLevel then
			BEC.OverrideLevel = level
		end
	else
		print("Huh?")
	end
end

function BEC.CommandHandler(msg, editBox)
	local command, rest = msg:match("^(%S*)%s*(.-)$")

	BEC:DebugPrint("Command: " .. command)
	BEC:DebugPrint("Rest:" .. rest)

	if strlower(command) == "checkthorns" and rest ~= "" then
		if strlower(rest) == "true" then
			BuffEnoughClassic_Settings.ShouldCheckThorns = true
			print("BEC: Thorns will be checked where available.")
		elseif strlower(rest) == "false" then
			BuffEnoughClassic_Settings.ShouldCheckThorns = false
			print("BEC: Thorns will not be checked.")
		else
			print("Unknown Parameter for 'checkthorns': " .. rest)
			print("Acceptable values are 'true' or 'false'.")
		end	
	elseif strlower(command) == "ignoreoptional" and rest ~= "" then
		if strlower(rest) == "true" then
			BuffEnoughClassic_Settings.IgnoreOptional = true
			print("BEC: Optional Buffs will not be checked.")
		elseif strlower(rest) == "false" then
			BuffEnoughClassic_Settings.IgnoreOptional = false
			print("BEC: Optional Buffs will be checked.")
		else
			print("Unknown Parameter for 'ignoreoptional': " .. rest)
			print("Acceptable values are 'true' or 'false'.")
		end
	elseif strlower(command) == "ignoreweak" and rest ~= "" then
		if strlower(rest) == "true" then
			BuffEnoughClassic_Settings.IgnoreWeak = true
			print("BEC: Weak Buffs will not be checked.")
		elseif strlower(rest) == "false" then
			BuffEnoughClassic_Settings.IgnoreWeak = false
			print("BEC: Weak Buffs will be checked.")
		else
			print("Unknown Parameter for 'ignoreweak': " .. rest)
			print("Acceptable values are 'true' or 'false'.")
		end
	elseif strlower(command) == "locked" and rest ~= "" then
		if strlower(rest) == "true" then
			BuffEnoughClassic_Settings.Locked = true
			BEC:RefreshLockStatus()
			print("BEC: Indicator Window locked.")
		elseif strlower(rest) == "false" then
			BuffEnoughClassic_Settings.Locked = false
			BEC:RefreshLockStatus()
			print("BEC: Indicator Window unlocked.")
		else
			print("Unknown Parameter for 'locked': " .. rest)
			print("Acceptable values are 'true' or 'false'.")
		end
	elseif strlower(command) == "size" and rest ~= "" then
		if strlower(rest) == "small" then
			BuffEnoughClassic_Settings.Size = 0
			BEC:SetWindowSize(BuffEnoughClassic_Settings.Size)
		elseif strlower(rest) == "normal" then
			BuffEnoughClassic_Settings.Size = 1
			BEC:SetWindowSize(BuffEnoughClassic_Settings.Size)
		elseif strlower(rest) == "large" then
			BuffEnoughClassic_Settings.Size = 2
			BEC:SetWindowSize(BuffEnoughClassic_Settings.Size)
		else
			print("Unknown Parameter for 'size': " .. rest)
			print("Acceptable Values are 'small', 'normal', or 'large'.")
		end
	elseif strlower(command) == "fauxbuffer" and rest ~= "" then
		BEC.ParseFauxBufferRequest(rest)
	elseif strlower(command) == "clearfaux" then
		BEC.ClearFauxBuffers()
	elseif strlower(command) == "fauxlist" then
		BEC.DisplayFauxBuffers()
	elseif strlower(command) == "playeroverride" and rest ~= "" then
		BEC.ParsePlayerOverride(rest)
	elseif strlower(command) == "config" then
		print("===================")
		print("Buff Enough Classic Configuration:")
		print("Ignore Weak Buffs: " .. tostring(BuffEnoughClassic_Settings.IgnoreWeak))
		print("Ignore Optional Buffs: " .. tostring(BuffEnoughClassic_Settings.IgnoreOptional))
		print("Check Thorns: " .. tostring(BuffEnoughClassic_Settings.ShouldCheckThorns))

		if BuffEnoughClassic_Settings.Size < 1 then
			print("Size: Small")
		elseif BuffEnoughClassic_Settings.Size == 1 then
			print("Size: Normal")
		else
			print("Size: Large")
		end

		print("Indicator Window Locked: ".. tostring(BuffEnoughClassic_Settings.Locked))

		BEC.DisplayFauxBuffers()
		print("===================")
	elseif strlower(command) == "debug" then
		if strlower(rest) == "buffs" then
			for i = 1, 40 do
				local name, _, _, _, _, expires, _, _, _, spellID = UnitBuff("player", i)

				if name ~= nil then
					print(name .. " ID:" .. spellID)
				end
			end
		elseif strlower(rest) == "talents" then
			BEC:DebugPrintTalents()
		elseif strlower(rest) == "group" then
			BEC:DebugPrintGroup()
		elseif strlower(rest) == "print" then
			BEC.Debug = not BEC.Debug

			print("Debugging Output: " .. tostring(BEC.Debug))
		elseif strlower(rest) == "updateprint" then
			BEC.DebugUpdate = not BEC.DebugUpdate

			print("Debugging Update Output: " .. tostring(BEC.DebugUpdate))
		end
	elseif strlower(command) == "help" or strlower(command) == "h" then
		PrintHelp()
	else
		print("Unknown Command: " .. command)
		print("Command Parameters: " .. rest)

		PrintHelp()
	end
end

SlashCmdList["BECCMD"] = BEC.CommandHandler
