-- Globals
BuffEnough = {}

local BEC = BuffEnough

BEC.Debug = false
BEC.DebugUpdate = false
BEC.FauxMembers = {}
BEC.Game_MaxLevel = 70

function BEC:DebugPrint(value)
	if BEC.Debug then
		print(value)
	end
end

function BEC:DebugUpdatePrint(value)
	if BEC.DebugUpdate then
		BEC:DebugPrint(value)
	end
end

-- DescendingSort
-- Description: Returns True if the first value is greater than the second.
-- Parameters:
-- first - The first value.
-- second - The second value.
function DescendingSort(first, second)
	return (first > second)
end

function BEC:SetWindowSize(size)
	if size < 1 then
		BuffEnough_Window:SetSize(24, 16)
	elseif size == 1 then
		BuffEnough_Window:SetSize(48, 32)
	else
		BuffEnough_Window:SetSize(96, 64)
	end
end

BEC:DebugPrint("Starting BuffEnough")

local lastUpdate = 0

local missingBuff = false
local wrongBuff = false
local missingBuffTooltip = ""

local BuffEnough_Window

-- CheckPlayerForBuff
-- Description: Checks whether the player has one of the set of required buffs,
--				and returns True if any one of them is found.
-- Parameters:
-- spellIDsRequired - Array of spell ID's to check for.
local function CheckPlayerForBuff(spellIDsRequired)
	for i = 1, 40 do
		local name, _, _, _, _, expires, _, _, _, spellID = UnitBuff("player", i)

		if name ~= nil then
			if BEC:SetContains(spellIDsRequired, spellID) then
				local remains = 0

				if expires > 0 then
					remains = expires - GetTime()

					if remains > 0 then
						return true
					end
				else
					return true
				end
			end
		end
	end

	return false
end

function BEC:GatherPlayerBuffs()
	local rc = {}

	for i = 1, 40 do
		local name, _, _, _, _, expires, _, _, _, spellID = UnitBuff("player", i)

		table.insert(rc, {["SpellID"] = spellID, ["Expiry"] = expires, ["Name"] = name})
	end

	return rc
end

function BEC:GetPlayerClass()
	if BEC.OverrideClass ~= nil and BEC.OverrideClassID > 0 then
		return BEC.OverrideClass, "UNKNOWN", BEC.OverrideClassID
	end
	return UnitClass("player")
end

function BEC:GetPlayerLevel()
	if BEC.OverrideLevel ~= nil then
		if BEC.OverrideLevel > 0 and BEC.OverrideLevel <= BEC.Game_MaxLevel then
			return BEC.OverrideLevel
		end
	end
	return UnitLevel("player")
end

function BEC:RefreshLockStatus()
	if BuffEnoughClassic_Settings.Locked then
		BuffEnough_Window:RegisterForDrag()
	else
		BuffEnough_Window:RegisterForDrag("LeftButton")
	end
end

function BuffBox_OnEnter()
	GameTooltip_SetDefaultAnchor(GameTooltip, UIParent)
	GameTooltip:SetText(missingBuffTooltip)
	GameTooltip:Show()
end

function BEC:AddFauxBuffer(class, classID, level)
	table.insert(BEC.FauxMembers, {["Level"] = level, ["ClassID"] = classID, ["Class"] = class, ["Name"] = "Faux " .. class})
end

function BEC:DisplayFauxBuffers()
	if BEC.FauxMembers ~= nil then
		if table.getn(BEC.FauxMembers) > 0 then
			local fauxList = BEC.FauxMembers

			print("Faux Buffers")
			for i = 1, #fauxList do
				local fauxMember = fauxList[i]

				print(fauxMember.Name .. "(" .. fauxMember.Level .. ")")
			end
		else
			print("No faux buffers.")
		end
	end
end

function BEC:ClearFauxBuffers()
	BEC.FauxMembers = {}
end

function BEC:GetGroupComposition()
	local rc = {}
	local numGroupMembers = GetNumGroupMembers()

	rc.MaxMageLevel = 0
	rc.MaxDruidLevel = 0
	rc.MaxPriestLevel = 0
	rc.PaladinLevels = {}
	rc.Members = {}
	rc.PlayerLevel = BEC.GetPlayerLevel()
	rc.PlayerClass, _, rc.PlayerClassID = BEC.GetPlayerClass()
	rc.PlayerSpecID = BEC.PlayerSpecID

	BEC:DebugUpdatePrint("Class: " .. rc.PlayerClass .. " ID: " .. rc.PlayerClassID)

	if IsInRaid() then
		for i = 1, numGroupMembers do
			local unit = "raid" .. i

			if UnitExists(unit) then
				local name = UnitName(unit)
				local class, _, classID = UnitClass(unit)
				local level = UnitLevel(unit)

				if classID ~= nil then
					BEC:DebugUpdatePrint("Unit: " .. unit .. " Class: " .. class .. " Class ID: " .. classID .. " Level: " .. level)
				else
					print("Unit:" .. unit .. " had a nil classID.")
				end

				if (classID > 0) then
					if classID == BEC.Class.Druid then
						if level > rc.MaxDruidLevel then
							BEC:DebugUpdatePrint("Updating Max Druid Level to " .. level)
							rc.MaxDruidLevel = level
						end
					elseif classID == BEC.Class.Mage then
						if level > rc.MaxMageLevel then
							rc.MaxMageLevel = level
						end
					elseif classID == BEC.Class.Priest then
						if level > rc.MaxPriestLevel then
							rc.MaxPriestLevel = level
						end
					elseif classID == BEC.Class.Paladin then
						table.insert(rc.PaladinLevels, level)
					end

					if name ~= nil and not name == "" then
						rc.Members[name] = {["Level"] = level, ["ClassID"] = classID, ["Class"] = class, ["Name"] = name}
					end
				end
			end
		end
	else
		-- Add the player to the members.  We don't need to do it in raid because the player is part of the raid units.
		rc.Members[UnitName("player")] = {["Level"] = rc.PlayerLevel, ["ClassID"] = rc.PlayerClassID, ["Class"] = rc.PlayerClass, ["Name"] = UnitName("player")}

		if rc.PlayerClassID == BEC.Class.Druid then
			rc.MaxDruidLevel = rc.PlayerLevel
		elseif rc.PlayerClassID == BEC.Class.Mage then
			rc.MaxMageLevel = rc.PlayerLevel
		elseif rc.PlayerClassID == BEC.Class.Priest then
			rc.MaxPriestLevel = rc.PlayerLevel
		elseif rc.PlayerClassID == BEC.Class.Paladin then
			table.insert(rc.PaladinLevels, rc.PlayerLevel)
		end

		if IsInGroup() then
			for i = 1, numGroupMembers - 1 do
				local unit = "party" .. i

				if UnitExists(unit) then
					local name = UnitName(unit)
					local class, _, classID = UnitClass(unit)
					local level = UnitLevel(unit)

					if classID ~= nil then
						if classID == BEC.Class.Druid then
							if level > rc.MaxDruidLevel then
								rc.MaxDruidLevel = level
							end
						elseif classID == BEC.Class.Mage then
							if level > rc.MaxMageLevel then
								rc.MaxMageLevel = level
							end
						elseif classID == BEC.Class.Priest then
							if level > rc.MaxPriestLevel then
								rc.MaxPriestLevel = level
							end
						elseif classID == BEC.Class.Paladin then
							table.insert(rc.PaladinLevels, level)
						end
					end

					rc.Members[name] = {["Level"] = level, ["ClassID"] = classID, ["Class"] = class, ["Name"] = name}
				end
			end
		end
	end

	if table.getn(BEC.FauxMembers) > 0 then
		local fauxList = BEC.FauxMembers

		for i = 1, #fauxList do
			local fauxMember = fauxList[i]
			rc.Members[fauxMember.Name .. i] = fauxMember

			if fauxMember.ClassID == BEC.Class.Druid then
				if fauxMember.Level > rc.MaxDruidLevel then
					rc.MaxDruidLevel = fauxMember.Level
				end
			elseif fauxMember.ClassID == BEC.Class.Mage then
				if fauxMember.Level > rc.MaxMageLevel then
					rc.MaxMageLevel = fauxMember.Level
				end
			elseif fauxMember.ClassID == BEC.Class.Priest then
				if fauxMember.Level > rc.MaxPriestLevel then
					rc.MaxPriestLevel = fauxMember.Level
				end
			elseif fauxMember.ClassID == BEC.Class.Paladin then
				table.insert(rc.PaladinLevels, fauxMember.Level)
			end

			BEC:DebugUpdatePrint("Unit: Faux" .. fauxMember.Class .. " Class: " .. fauxMember.Class .. " Class ID: " .. fauxMember.ClassID)
		end
	end

	-- Sort the paladin levels.
	if table.getn(rc.PaladinLevels) > 0 then
		table.sort(rc.PaladinLevels, DescendingSort)
	end

	return rc
end

function BEC:DebugPrintGroup()
	local groupComp = BEC:GetGroupComposition()
	local paladinLevelString = ""

	print("Group Composition")

	print("Mage Level: " .. groupComp.MaxMageLevel)
	print("Druid Level: " .. groupComp.MaxDruidLevel)
	print("Priest Level: " .. groupComp.MaxPriestLevel)

	for i = 1, table.getn(groupComp.PaladinLevels) do
		if i == 1 then
			paladinLevelString = groupComp.PaladinLevels[i]
		else
			paladinLevelString = paladinLevelString .. ", " .. groupComp.PaladinLevels[i]
		end
	end
	print("Paladin Levels: " .. paladinLevelString)
	print("Paladin Levels: " .. BEC:ToCsv(groupComp.PaladinLevels))

	print("Members (" .. table.getn(groupComp.Members) .. ")")
	print("=============")

	for name, member in pairs(groupComp.Members) do
		print(member.Name .. " (" .. member.Class .. " " .. member.Level .. ")")
	end
end

function BuffBox_OnLeave()
	GameTooltip:Hide()
end

function BuffBox_OnDragStart(self)
	BEC:DebugPrint("Starting Drag")
	self:StartMoving()
end

function BuffBox_OnDragStop(self)
	self:StopMovingOrSizing()

	BuffEnoughClassic_Settings.XPos = self:GetLeft()
	BuffEnoughClassic_Settings.YPos = self:GetBottom()

	BEC:DebugPrint("Set Position " .. BuffEnoughClassic_Settings.XPos .. "," .. BuffEnoughClassic_Settings.YPos)
end

function BuffBox_OnLoad(self)
	--self:RegisterForDrag("LeftButton");
	--self:RegisterEvent("PLAYER_ENTERING_WORLD")
end

function BEC:RefreshClassTalentBuffs()
	local _, _, classID = BEC:GetPlayerClass()
	local level = BEC:GetPlayerLevel()

	BEC.PlayerClassBuffs = BEC:GetPlayerClassBuffs(classID)
	BEC.PlayerSpecID, BEC.PlayerTalentedBuffs = BEC:GetPlayerSpecAndTalentedBuffs(classID, level)
end

function BuffBox_OnEvent(self, event, ...)
	if event == "PLAYER_ENTERING_WORLD" then
		BEC:DebugPrint("Loading Settings")

		BuffEnoughClassic_Settings = BuffEnoughClassic_Settings or {}

		-- Debug for optional Settings
		if BuffEnoughClassic_Settings.ShouldCheckThorns == nil then
			BuffEnoughClassic_Settings.ShouldCheckThorns = true
		end

		if BuffEnoughClassic_Settings.IgnoreOptional == nil then
			BuffEnoughClassic_Settings.IgnoreOptional = false
		end

		if BuffEnoughClassic_Settings.IgnoreWeak == nil then
			BuffEnoughClassic_Settings.IgnoreWeak = false
		end

		if BuffEnoughClassic_Settings.Size == nil then
			BuffEnoughClassic_Settings.Size = 1
		end

		if BuffEnoughClassic_Settings.Locked == nil then
			BuffEnoughClassic_Settings.Locked = false
		end

		BEC:SetWindowSize(BuffEnoughClassic_Settings.Size)
		BEC:RefreshLockStatus()

		if BuffEnoughClassic_Settings.XPos then
			BEC:DebugPrint("Loading Position " .. BuffEnoughClassic_Settings.XPos .. "," .. BuffEnoughClassic_Settings.YPos)

			self:ClearAllPoints()
			self:SetPoint("BOTTOMLEFT", BuffEnoughClassic_Settings.XPos, BuffEnoughClassic_Settings.YPos)
		end

		BEC:RefreshClassTalentBuffs()
	elseif ((event == "CHARACTER_POINTS_CHANGED") or (event == "SPELLS_CHANGED") or (event == "PLAYER_LEVEL_CHANGED") or (event == "PLAYER_LEVEL_UP")) then
		BEC:RefreshClassTalentBuffs()
	end
end

-- Returns:
-- (Boolean) Spell is found.
-- (Boolean) Spell is with appropriate levels.
-- (Boolean) Spell name.
function BEC:CheckBuffFromList(playerBuffs, casterLevel, spellList, defaultName, ignoreBuffSpellIDs)
	for i = 1, #spellList do
		local hasBuff, foundSpellID, foundBuffName = BEC:BuffListContainsSpellID(playerBuffs, spellList[i].SpellIDs, ignoreBuffSpellIDs)

		if hasBuff then
			table.insert(ignoreBuffSpellIDs, foundSpellID)

			if spellList[i].Until <= casterLevel then
				-- The spell we found is lower than the caster level.
				return true, false, foundBuffName
			else
				-- The spell is at or higher than the caster level so we're good
				return true, true, foundBuffName
			end
		end
	end
	-- We didn't find the spell.
	return false, false, defaultName
end

function BuffBox_OnUpdate(self, elapsed, ...)
	local curTime = GetTime()

	-- Only update every half a second.
	if (lastUpdate + 0.5) < curTime then
		local groupComp = BEC:GetGroupComposition()

		BEC:DebugUpdatePrint("Update At " .. curTime)

		local missingBuff = false
		local wrongBuff = false
		local missingOptionalBuff = false
		local weakBuff = false

		missingBuffTooltip = "Missing Buffs\r\n====================="

		if BEC.Debug then
			missingBuffTooltip = missingBuffTooltip .. "\r\nPlayer Spec: " .. BEC.PlayerSpecID
		end

		local playerBuffs = BEC:GatherPlayerBuffs()
		local groupSpellLists = BEC:GetGroupBuffSetup(groupComp)

		if groupSpellLists ~= nil and #groupSpellLists > 0 then
			for i = 1, #groupSpellLists do
				local casters = groupSpellLists[i].CasterLevels
				local ignoreBuffSpellIDs = {}
				local missingCount = 0

				table.sort(casters, DescendingSort)

				for c = 1, min(#casters, groupSpellLists[i].MaxNeeded) do
					local found, appropriate, buffName =
						BEC:CheckBuffFromList(playerBuffs, casters[c], groupSpellLists[i].SpellList, groupSpellLists[i].Name, ignoreBuffSpellIDs)

					if found then
						if not appropriate then
							missingBuffTooltip = missingBuffTooltip .. "\r\n" .. buffName
							missingBuffTooltip = missingBuffTooltip .. " (Weak - Caster: " .. casters[c] .. ")"
							weakBuff = true
						end
					else
						missingCount = missingCount + 1
					end
				end

				if missingCount > 0 then
					missingBuffTooltip = missingBuffTooltip .. "\r\n" .. groupSpellLists[i].Name

					-- If we're missing more than one, list the number of them.
					if missingCount > 1 then
						missingBuffTooltip = missingBuffTooltip .. " (" .. missingCount .. ")"
					end

					missingBuff = true
				end
			end
		end

		if BEC.PlayerTalentedBuffs ~= nil then
			local talentBuffs = BEC.PlayerTalentedBuffs

			for i = 1, #talentBuffs do
				if CheckPlayerForBuff(talentBuffs[i].SpellIDs) == false then
					missingBuffTooltip = missingBuffTooltip .. "\r\n" .. talentBuffs[i].DisplayText

					if talentBuffs[i].Required then
						missingBuff = true
					else
						missingOptionalBuff = true
						missingBuffTooltip = missingBuffTooltip .. " (Optional)"
					end

					if BEC.Debug then
						missingBuffTooltip = missingBuffTooltip .. " (" .. BEC:ToCsv(talentBuffs[i].SpellIDs) .. ")"
					end
				end
			end
		end

		if BEC.PlayerClassBuffs ~= nil then
			local playerClassBuffs = BEC.PlayerClassBuffs

			for i = 1, #playerClassBuffs do
				local ignoreBuffSpellIDs = {}
				local found, appropriate, buffName =
					BEC:CheckBuffFromList(playerBuffs, groupComp.PlayerLevel, playerClassBuffs[i].SpellList, playerClassBuffs[i].Name, ignoreBuffSpellIDs)

				if found then
					if not appropriate then
						missingBuffTooltip = missingBuffTooltip .. "\r\n" .. buffName

						if playerClassBuffs[i].Optional then
							missingBuffTooltip = missingBuffTooltip .. " (Optional)"
						end

						missingBuffTooltip = missingBuffTooltip .. " (Weak - Caster: " .. groupComp.PlayerLevel .. ")"
						weakBuff = true
					end
				else
					missingBuffTooltip = missingBuffTooltip .. "\r\n" .. buffName
					
					if playerClassBuffs[i].Optional then
						missingOptionalBuff = true
						missingBuffTooltip = missingBuffTooltip .. " (Optional)"
					else						
						missingBuff = true
					end
				end
			end
		end		

		-- Remember the last time we updated.
		lastUpdate = curTime

		if missingBuff then
			self.tex:SetColorTexture(1.0, 0, 0)
		elseif (weakBuff and BuffEnoughClassic_Settings.IgnoreWeak ~= true) then
			self.tex:SetColorTexture(0.5, 0.2, 0.5)
		elseif (missingOptionalBuff and BuffEnoughClassic_Settings.IgnoreOptional ~= true) then
			self.tex:SetColorTexture(0.5, 0.5, 0)
		else
			--BEC:DebugPrint("No Missing Buffs")
			missingBuffTooltip = "No missing buffs!"
			self.tex:SetColorTexture(0, 0.4, 0)
		end

		BEC:DebugUpdatePrint(missingBuffTooltip)
	end
end

local function init()
	BuffEnough_Window = CreateFrame("Frame", "BuffEnough_Window", UIParent)
	BuffEnough_Window:SetPoint("BOTTOMLEFT")
	BuffEnough_Window:SetSize(48, 32)
	BuffEnough_Window:EnableMouse(true)
	BuffEnough_Window:SetMovable(true)
	BuffEnough_Window:SetScript("OnLoad", BuffBox_OnLoad)
	BuffEnough_Window:SetScript("OnEvent", BuffBox_OnEvent)
	BuffEnough_Window:SetScript("OnUpdate", BuffBox_OnUpdate)
	BuffEnough_Window:SetScript("OnDragStart", BuffBox_OnDragStart)
	BuffEnough_Window:SetScript("OnDragStop", BuffBox_OnDragStop)
	BuffEnough_Window:SetScript("OnEnter", BuffBox_OnEnter)
	BuffEnough_Window:SetScript("OnLeave", BuffBox_OnLeave)
	BuffEnough_Window:RegisterForDrag("LeftButton")
	BuffEnough_Window:RegisterEvent("PLAYER_ENTERING_WORLD")
	BuffEnough_Window:RegisterEvent("CHARACTER_POINTS_CHANGED")
	BuffEnough_Window:RegisterEvent("SPELLS_CHANGED")
	BuffEnough_Window:RegisterEvent("PLAYER_LEVEL_CHANGED")
	BuffEnough_Window:RegisterEvent("PLAYER_LEVEL_UP")

	BuffEnough_Window.tex = BuffEnough_Window:CreateTexture()
	BuffEnough_Window.tex:SetAllPoints(BuffEnough_Window)
	BuffEnough_Window.tex:SetColorTexture(1.0, 0, 0)

	BEC.Window = BuffEnough_Window
end

-- Start the Addon.
init()

BEC:DebugPrint("BuffEnough Setup complete")
