local BEC = BuffEnough

local KeyTalentTable = {
	[BEC.ClassSpec.DruidBalance] = "Moonkin Form",
	[BEC.ClassSpec.DruidFeral] = "Leader of the Pack",
	[BEC.ClassSpec.DruidRestoration] = "Gift of Nature",
	[BEC.ClassSpec.HunterBeastMastery] = "Beastial Wrath",
	[BEC.ClassSpec.HunterMarksmanship] = "Trueshot Aura",
	[BEC.ClassSpec.HunterSurvival] = "Wyvern Sting",
	[BEC.ClassSpec.MageArcane] = "Arcane Power",
	[BEC.ClassSpec.MageFire] = "Combustion",
	[BEC.ClassSpec.MageFrost] = "Ice Barrier",
	[BEC.ClassSpec.PaladinHoly] = "Holy Guidance",
	[BEC.ClassSpec.PaladinProtection] = "Improved Righteous Fury",
	[BEC.ClassSpec.PaladinRetribution] = "Crusader Strike",
	[BEC.ClassSpec.PriestDiscipline] = "Power Infusion",
	[BEC.ClassSpec.PriestHoly] = "Spirit of Redemption",
	[BEC.ClassSpec.PriestShadow] = "Vampiric Embrace",
	[BEC.ClassSpec.RogueAssassination] = "Vigor",
	[BEC.ClassSpec.RogueCombat] = "Adrenaline Rush",
	[BEC.ClassSpec.RogueSubtlety] = "Premeditation",
	[BEC.ClassSpec.ShamanElemental] = "Elemental Mastery",
	[BEC.ClassSpec.ShamanEnhancement] = "Dual Wield",
	[BEC.ClassSpec.ShamanRestoration] = "Earth Shield",
	[BEC.ClassSpec.WarlockAffliction] = "Contagion",
	[BEC.ClassSpec.WarlockDemonology] = "Summon Felguard",
	[BEC.ClassSpec.WarlockDestruction] = "Backlash",
	[BEC.ClassSpec.WarriorArms] = "Mortal Strike",
	[BEC.ClassSpec.WarriorFury] = "Bloodthirst",
	[BEC.ClassSpec.WarriorProtection] = "Shield Slam"
}

BEC.TalentBuffLevels = {}

BEC.TalentBuffLevels.Druid = {
	{
		["TalentName"]="Tree of Life",
		["Levels"]={
			{["Level"]=50,["SpellIDs"]={33891},["Required"]=false,["DisplayText"]="Tree Form"}
		}
	},
	{
		["TalentName"]="Moonkin Form",
		["Levels"]={
			{["Level"]=50,["SpellIDs"]={24858},["Required"]=false,["DisplayText"]="Moonkin Form"}
		}
	},
	{
		["TalentName"]="Omen of Clarity",
		["Levels"]={
			{["Level"]=20,["SpellIDs"]={16864},["Required"]=true,["DisplayText"]="Omen of Clarity"}
		}
	},
	{
		["TalentName"]="Leader of the Pack",
		["Levels"]={
			{["Level"]=40,["SpellIDs"]={24932},["Required"]=false,["DisplayText"]="Leader of the Pack"}
		}
	}
}

BEC.TalentBuffLevels.Hunter = {
	{
		["TalentName"]="Trueshot Aura",
		["Levels"]={
			{["Level"]=40,["SpellIDs"]={19506},["Required"]=true,["DisplayText"]="Trueshot Aura"},
			{["Level"]=50,["SpellIDs"]={20905},["Required"]=true,["DisplayText"]="Trueshot Aura"},
			{["Level"]=60,["SpellIDs"]={20906},["Required"]=true,["DisplayText"]="Trueshot Aura"},
			{["Level"]=60,["SpellIDs"]={27066},["Required"]=true,["DisplayText"]="Trueshot Aura"}
		}
	}
}

BEC.TalentBuffLevels.Mage = {}

BEC.TalentBuffLevels.Paladin = {
	-- Since we can't require all Paladins to use Righteous Fury, we will only require it
	-- when the player has the Improved Righteous Fury talent.  This is unlikely to be
	-- a purchased talent for non-tanks.  It will mean that there's a gap of at least 5 
	-- levels between 16 and 21 where the player won't be prompted, but there's no way 
	-- to determine it otherwise.
	["TalentName"]="Improved Righteous Fury",
	["Levels"]={
		{["Level"]=16,["SpellIDs"]={25780},["Required"]=true,["DisplayText"]="Righteous Fury"}
	}
}

BEC.TalentBuffLevels.Priest = {
	{
		["TalentName"]="Shadowform",
		["Levels"]={
			{["Level"]=40,["SpellIDs"]={15473},["Required"]=true,["DisplayText"]="Shadowform"}
		}
	}
}

BEC.TalentBuffLevels.Rogue = {}

BEC.TalentBuffLevels.Shaman = {}

BEC.TalentBuffLevels.Warlock = {
	{
		["TalentName"]="Soul Link",
		["Levels"]={
			{["Level"]=40,["SpellIDs"]={25228},["Required"]=true,["DisplayText"]="Soul Link"}
		}
	}
}

BEC.TalentBuffLevels.Warrior = {}

local function CreateTalentTable()
	local rc = {}
		
	local numTabs = GetNumTalentTabs();
	for t=1, numTabs do
		local numTalents = GetNumTalents(t);
		for i=1, numTalents do
			nameTalent, icon, tier, column, currRank, maxRank= GetTalentInfo(t,i);

			if currRank > 0 then
				table.insert(rc, nameTalent)
			end
		end
	end
	return rc
end

function BEC:DebugPrintTalents()	
	print("Talents")

	local numTabs = GetNumTalentTabs();
	for t=1, numTabs do
		local numTalents = GetNumTalents(t);
		for i=1, numTalents do
			nameTalent, icon, tier, column, currRank, maxRank= GetTalentInfo(t,i);
			
			print("- "..nameTalent..": "..currRank.."/"..maxRank);
		end
	end
end

local function AddSpellForTalent(playerLevel, talentTable, spellDetails, talentBuffs)
	if spellDetails ~= nil then
		for spellCounter = 1, #spellDetails do
			if BEC:SetContains(talentTable, spellDetails[spellCounter].TalentName) then
				local entry = nil
				local levels = spellDetails[spellCounter].Levels

				for i=1, #levels do
					-- Tables should be sorted in level ascending order, so just record 
					-- the appropriate entry and the last one will be the highest.
					if levels[i].Level <= playerLevel then
						entry = levels[i]
					end
				end

				if entry ~= nil then
					table.insert(talentBuffs, {["SpellIDs"]=entry.SpellIDs, ["Required"]=entry.Required, ["DisplayText"]=entry.DisplayText})
				end
			end
		end
	end
end


local function GetTalentedBuffs(classID,level,talentTable)
	local rc = {}

	if classID == BEC.Class.Druid then
		AddSpellForTalent(level, talentTable, BEC.TalentBuffLevels.Druid, rc)
	elseif classID == BEC.Class.Hunter then
		AddSpellForTalent(level, talentTable, BEC.TalentBuffLevels.Hunter, rc)
	elseif classID == BEC.Class.Mage then
		AddSpellForTalent(level, talentTable, BEC.TalentBuffLevels.Mage, rc)
	elseif classID == BEC.Class.Paladin then
		AddSpellForTalent(level, talentTable, BEC.TalentBuffLevels.Paladin, rc)
	elseif classID == BEC.Class.Priest then
		AddSpellForTalent(level, talentTable, BEC.TalentBuffLevels.Priest, rc)
	elseif classID == BEC.Class.Rogue then
		AddSpellForTalent(level, talentTable, BEC.TalentBuffLevels.Rogue, rc)
	elseif classID == BEC.Class.Shaman then
		AddSpellForTalent(level, talentTable, BEC.TalentBuffLevels.Shaman, rc)
	elseif classID == BEC.Class.Warlock then
		AddSpellForTalent(level, talentTable, BEC.TalentBuffLevels.Warlock, rc)
	elseif classID == BEC.Class.Warrior then
		AddSpellForTalent(level, talentTable, BEC.TalentBuffLevels.Warrior, rc)
	end

	BEC:DebugPrint("Printing "..#rc.." Talented Buffs")

	if #rc > 0 then
		for i=1, #rc do
			BEC:DebugPrint("Name:"..rc[i].DisplayText.." Spell IDs:"..tostring(rc[i].SpellIDs).." Required:"..tostring(rc[i].Required))
		end
	else
		BEC:DebugPrint("No talented buffs.")
	end

	return rc
end

local function GetClassSpec(classID, talentTable)
	local rc = 0

	for i=1, #KeyTalentTable do
		if BEC:SetContains(talentTable,KeyTalentTable[i]) then
			rc = i
			
			break
		end
	end

	BEC:DebugPrint("Your Spec is "..rc)

	return rc
end

function BEC:GetPlayerSpecAndTalentedBuffs(classID,level)
	local talentTable = CreateTalentTable();

	local spec = GetClassSpec(classID, talentTable)
	local talentBuffs = GetTalentedBuffs(classID, level, talentTable)

	return spec, talentBuffs
end
