local BEC = BuffEnough

-- assumes the tbl is an array, i.e., all the keys are
-- successive integers - otherwise #tbl will fail
function BEC:enum(tbl)
    local length = #tbl
    for i = 1, length do
        local v = tbl[i]
        tbl[v] = i
    end

    return tbl
end

BEC.Class = {
    ["Druid"] = 11,
    ["Hunter"] = 3,
    ["Mage"] = 8,
    ["Paladin"] = 2,
    ["Priest"] = 5,
    ["Rogue"] = 4,
    ["Shaman"] = 7,
    ["Warlock"] = 9,
    ["Warrior"] = 1,
    ["DeathKnight"] = 6, -- Future Proofing
    ["Monk"] = 10,
    ["DemonHunter"] = 12
}

BEC.ClassSpec =
    BEC:enum(
    {
        "DruidBalance",
        "DruidFeral",
        "DruidRestoration",
        "HunterBeastMastery",
        "HunterMarksmanship",
        "HunterSurvival",
        "MageArcane",
        "MageFire",
        "MageFrost",
        "PaladinHoly",
        "PaladinProtection",
        "PaladinRetribution",
        "PriestDiscipline",
        "PriestHoly",
        "PriestShadow",
        "RogueAssassination",
        "RogueCombat",
        "RogueSubtlety",
        "ShamanElemental",
        "ShamanEnhancement",
        "ShamanRestoration",
        "WarlockAffliction",
        "WarlockDemonology",
        "WarlockDestruction",
        "WarriorArms",
        "WarriorFury",
        "WarriorProtection"
    }
)

-- Class names. Can be changed for localization.
local Druid = "Druid"
local Hunter = "Hunter"
local Mage = "Mage"
local Priest = "Priest"
local Paladin = "Paladin"
local Rogue = "Rogue"
local Shaman = "Shaman"
local Warlock = "Warlock"
local Warrior = "Warrior"

-- Utility Methods

-- GetHighest
-- Description: Return the highest value from the array.
-- Parameters:
-- - values - The array to retrieve a value from.
-- - skip - Ignores the highest 'skip' number of entries.
function BEC:GetHighest(values, skip)
    if values ~= nil then
        local length = table.getn(values)

        if length > skip then
            -- Zero based array so the last entry is length-1.
            -- To skip 1 entry it should return length-2.
            return values[length - 1 - skip]
        else
            return nil
        end
    end
end

function BEC:SplitString(inputstr, sep)
    if sep == nil then
        sep = "%s"
    end
    local t = {}
    for str in string.gmatch(inputstr, "([^" .. sep .. "]+)") do
        table.insert(t, str)
    end
    return t
end

-- DescendingSort
-- Description: Returns True if the first value is greater than the second.
-- Parameters:
-- first - The first value.
-- second - The second value.
function BEC:DescendingSort(first, second)
    return (first > second)
end

-- SetContains
-- Description: Returns True if the provided set contains the value, or the value is nil.
-- Parameters:
-- set - The set to check.
-- value - The value to check for.
function BEC:SetContains(set, value)
    if set ~= nil then
        for i, entry in ipairs(set) do
            if entry == value then
                return true
            end
        end
    else
        -- We have all the required buffs for a blank set.
        return true
    end

    return false
end

function BEC:BuffListContainsSpellID(buffList, spellIDs, ignoreBuffSpellIDs)
    for i = 1, #buffList do
        -- Ignore the buffs we've already used.
        if not BEC:SetContains(ignoreBuffSpellIDs, buffList[i].SpellID) then
            -- Check if the buff is one of the list.
            if BEC:SetContains(spellIDs, buffList[i].SpellID) then
                return true, buffList[i].SpellID, buffList[i].Name
            end
        end
    end

    return false, 0, "Unknown"
end

function BEC:GetClassID(className)
    if strlower(className) == strlower(Druid) then
        return BEC.Class.Druid
    elseif strlower(className) == strlower(Hunter) then
        return BEC.Class.Hunter
    elseif strlower(className) == strlower(Mage) then
        return BEC.Class.Mage
    elseif strlower(className) == strlower(Priest) then
        return BEC.Class.Priest
    elseif strlower(className) == strlower(Paladin) then
        return BEC.Class.Paladin
    elseif strlower(className) == strlower(Rogue) then
        return BEC.Class.Rogue
    elseif strlower(className) == strlower(Shaman) then
        return BEC.Class.Shaman
    elseif strlower(className) == strlower(Warlock) then
        return BEC.Class.Warlock
    elseif strlower(className) == strlower(Warrior) then
        return BEC.Class.Warrior
    else
        return 0
    end
end

function BEC:AddSetToSet(base, addition)
    for i = 1, #addition do
        table.insert(base, addition[i])
    end
end

function BEC:HasMinimumRemaining(expires)
    if BEC.MinimumExpiry ~= nil then
        if (expires - GetTime()) >= BEC.MinimumExpiry then
            return true
        else
            return false
        end
    else
        return true
    end
end

function BEC:ToCsv(setValues)
    rc = ""

    if setValues ~= nil and #setValues > 0 then
        for i = 1, #setValues do
            if i > 1 then
                rc = rc .. "," .. setValues[i]
            else
                rc = rc .. setValues[i]
            end
        end
    end

    return rc
end
